#  """
#    Copyright (c) 2016- 2022, Wiliot Ltd. All rights reserved.
#
#    Redistribution and use of the Software in source and binary forms, with or without modification,
#     are permitted provided that the following conditions are met:
#
#       1. Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#
#       2. Redistributions in binary form, except as used in conjunction with
#       Wiliot's Pixel in a product or a Software update for such product, must reproduce
#       the above copyright notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the distribution.
#
#       3. Neither the name nor logo of Wiliot, nor the names of the Software's contributors,
#       may be used to endorse or promote products or services derived from this Software,
#       without specific prior written permission.
#
#       4. This Software, with or without modification, must only be used in conjunction
#       with Wiliot's Pixel or with Wiliot's cloud service.
#
#       5. If any Software is provided in binary form under this license, you must not
#       do any of the following:
#       (a) modify, adapt, translate, or create a derivative work of the Software; or
#       (b) reverse engineer, decompile, disassemble, decrypt, or otherwise attempt to
#       discover the source code or non-literal aspects (such as the underlying structure,
#       sequence, organization, ideas, or algorithms) of the Software.
#
#       6. If you create a derivative work and/or improvement of any Software, you hereby
#       irrevocably grant each of Wiliot and its corporate affiliates a worldwide, non-exclusive,
#       royalty-free, fully paid-up, perpetual, irrevocable, assignable, sublicensable
#       right and license to reproduce, use, make, have made, import, distribute, sell,
#       offer for sale, create derivative works of, modify, translate, publicly perform
#       and display, and otherwise commercially exploit such derivative works and improvements
#       (as applicable) in conjunction with Wiliot's products and services.
#
#       7. You represent and warrant that you are not a resident of (and will not use the
#       Software in) a country that the U.S. government has embargoed for use of the Software,
#       nor are you named on the U.S. Treasury Department’s list of Specially Designated
#       Nationals or any other applicable trade sanctioning regulations of any jurisdiction.
#       You must not transfer, export, re-export, import, re-import or divert the Software
#       in violation of any export or re-export control laws and regulations (such as the
#       United States' ITAR, EAR, and OFAC regulations), as well as any applicable import
#       and use restrictions, all as then in effect
#
#     THIS SOFTWARE IS PROVIDED BY WILIOT "AS IS" AND "AS AVAILABLE", AND ANY EXPRESS
#     OR IMPLIED WARRANTIES OR CONDITIONS, INCLUDING, BUT NOT LIMITED TO, ANY IMPLIED
#     WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, NONINFRINGEMENT,
#     QUIET POSSESSION, FITNESS FOR A PARTICULAR PURPOSE, AND TITLE, ARE DISCLAIMED.
#     IN NO EVENT SHALL WILIOT, ANY OF ITS CORPORATE AFFILIATES OR LICENSORS, AND/OR
#     ANY CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
#     OR CONSEQUENTIAL DAMAGES, FOR THE COST OF PROCURING SUBSTITUTE GOODS OR SERVICES,
#     FOR ANY LOSS OF USE OR DATA OR BUSINESS INTERRUPTION, AND/OR FOR ANY ECONOMIC LOSS
#     (SUCH AS LOST PROFITS, REVENUE, ANTICIPATED SAVINGS). THE FOREGOING SHALL APPLY:
#     (A) HOWEVER CAUSED AND REGARDLESS OF THE THEORY OR BASIS LIABILITY, WHETHER IN
#     CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE);
#     (B) EVEN IF ANYONE IS ADVISED OF THE POSSIBILITY OF ANY DAMAGES, LOSSES, OR COSTS; AND
#     (C) EVEN IF ANY REMEDY FAILS OF ITS ESSENTIAL PURPOSE.
#  """

# keys : macro name:
## duration [seconds]
## command:
###   gateway commands
###
##
"""
keys are the macro name
each macro contains:
    * command:
    ** if starts with !, it is gw command o send
    ** user_event, it will trigger user event with the specified text under 'values' key
    ** save log, it will save the log at the specified location under 'values' key
    * wait : time to wait in seconds after the command is done

"""

macros = {
    "update_version_macro": [{"command": "update_version", 'values': r'', "wait": 1}],
    "example_macro": [
        {"command": "!version", "wait": 3},
        {"command": "user_event", 'values': 'Set tag in initial location', "wait": 1},
        {"command": "!output_power pos3dBm", "wait": 5},
        {"command": "user_event", 'values': 'Pick up tag', "wait": 1},
        {"command": "!gateway_app 37 15 5 18", "wait": 3},
        {"command": "!cancel", "wait": 1},
        {"command": "!version", "wait": 1},
        {"command": "save_log", 'values': r'~/Downloads/output.csv', "wait": 1},

    ],
    "sub1g_calibration": [
        {"command": "!set_sub_1_ghz_power 29", "wait": 0},
        {"command": "!set_sub_1_ghz_energizing_frequency 915000", "wait": 0},
        {"command": "!beacons_backoff 0", "wait": 0},
        {"command": "user_event", 'values': 'Set new config', "wait": 0},
        {"command": "!gateway_app 37 15 5 50", "wait": 90},  # test for 90 seconds
        {"command": "!cancel", "wait": 1},
        {"command": "user_event", 'values': 'Stop gw app and eneter into brownout', "wait": 300},
        {"command": "save_log", 'values': r'~/Downloads/output.csv', "wait": 0},
    ],
    "toggle_rx_channel": [
        {"cyclic": True},
        {"command": "!scan_ch 37 37", "wait": 1},
        {"command": "!scan_ch 0 0", "wait": 1},
    ],
    "toggle_rx_advertising_channels": [
        {"cyclic": True},
        {"command": "!scan_ch 37", "wait": 1},
        {"command": "!scan_ch 38", "wait": 1},
        {"command": "!scan_ch 39", "wait": 1},
    ],
    "send_pulse": [
        {"cyclic": True},
        {"command": "!cmd_gpio SEND 1 P100 pulse 1 500", "wait": 1},
    ],

}
