empty_page = {
        "assembly_id": "1",
        "operator_name": "-",
        "inlay": "-",
        "assembled_reel_width": "",
        "#_of_lanes_in_antenna_design": "-",
        "assembled_reel_count": "-",
        "warning": "",
        "total_dies_qty_assembled": "0",
        "space": "",
        "tabs_1) wafer_qr_code": "",
        "tabs_1) temperature": "",
        "tabs_1) humidity": "",
        "tabs_1) date": "",
        "tabs_2) wafer_qr_code": "",
        "tabs_2) wafer_dies_qty_assembled": "",
        "tabs_2) temperature": "",
        "tabs_2) humidity": "",
        "tabs_2) date": "",
        "tabs_3) wafer_qr_code": "",
        "tabs_3) temperature": "",
        "tabs_3) humidity": "",
        "tabs_3) date": "",
        "tabs_4) wafer_qr_code": "",
        "tabs_4) wafer_dies_qty_assembled": "",
        "tabs_4) temperature": "",
        "tabs_4) humidity": "",
        "tabs_4) date": "",
        "tabs_5) wafer_qr_code": "",
        "tabs_5) temperature": "",
        "tabs_5) humidity": "",
        "tabs_5) date": "",
        "tabs_6) wafer_qr_code": "",
        "tabs_6) wafer_dies_qty_assembled": "",
        "tabs_6) temperature": "",
        "tabs_6) humidity": "",
        "tabs_6) date": "",
        "assembly_label": "",
        "wafers": "",
        "output": "",
    }

default_config = {
        "operator_name": ['Adrian', 'Liel', 'Yaniv', 'Yael'],
        "inlay": [117, 169],
        "#_of_lanes_in_antenna_design": [4, 5, 6],
        "assembled_reel_count": [1, 2, 3, 4, 5, 6],
        "printer_name": "",
        "inlay_rows": {
                "117": 5,
                "169": 4
            }
}